//=============================================================================
//  Midi Instrument Training plugin for MuseScore
//  Copyright (C) 2014 Jean-Baptiste Delisle
//  Based on colornotes plugin :
//  Copyright (C) 2012 Werner Schweer
//  Copyright (C) 2013 Nicolas Froment, Joachim Schmitz
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2
//  as published by the Free Software Foundation and appearing in
//  the file LICENCE.GPL
//=============================================================================

import QtQuick 2.0
import MuseScore 1.0
import QtQuick.Controls 1.0

MuseScore {
    property string pluginsPath: "/path/to/your/plugins/folder"
    property string midiPath: "/path/to/your/Midi/folder"
    property string defaultInstrument: "Your instrument"

    menuPath:   "Plugins.MidiInstrumentTraining"
    version:  "1.0"
    description: "This plugin takes input from a MIDI instrument and colors notes while you play from blue (note and rhythm are perfect) to red (bad note or bad rhythm)."

    pluginType: "dialog"
    width: 200
    height: 350

    onRun: {
        proc1.start(pluginsPath+"/MIT/qsynthconnect.sh "+instrumentName.text);
    }

    Column {
        spacing: units.gu(2)
        anchors.centerIn: parent
        Button {
            id: start
            anchors.horizontalCenter: parent.horizontalCenter
            text: "start"
            onClicked: {
                applyToNotesInSelection(uncolorNote);
                proc2.start(pluginsPath+"MIT/MIT "+instrumentName.text
                +" "+midiPath+curScore.name+".mid "
                +instrumentChannel.text+" "+scoreChannel.text+" "
                +max_badness.text+" "+weight_start.text+" "
                +weight_endp.text+" "+weight_endn.text);
            }
        }
        Button {
            id: stop
            anchors.horizontalCenter: parent.horizontalCenter
            text: "stop"
            onClicked: {
                proc2.kill();
                proc2.waitForFinished(-1);
                proc2.start(pluginsPath+"MIT/stopMIT");
            }
        }
        Text {
            anchors.horizontalCenter: parent.horizontalCenter
            text: "Score channel:"
        }
        TextField {
            id: scoreChannel
            anchors.horizontalCenter: parent.horizontalCenter
            width: 35
            text: "0"
        }

        Text {
            anchors.horizontalCenter: parent.horizontalCenter
            text: "MIDI Instrument name:"
        }
        TextField {
            id: instrumentName
            anchors.horizontalCenter: parent.horizontalCenter
            width: 100
            text: defaultInstrument
        }
        Text {
            anchors.horizontalCenter: parent.horizontalCenter
            text: "MIDI Instrument channel:"
        }
        TextField {
            id: instrumentChannel
            anchors.horizontalCenter: parent.horizontalCenter
            width: 35
            text: "0"
        }
        Button {
            id: synth
            anchors.horizontalCenter: parent.horizontalCenter
            text: "restart synth"
            onClicked: {
                proc1.kill();
                proc1.waitForFinished(-1);
                proc1.start(pluginsPath+"/MIT/qsynthconnect.sh "+instrumentName.text);
            }
        }
        Text {
            anchors.horizontalCenter: parent.horizontalCenter
            text: "WEIGHT START:"
        }
        TextField {
            id: weight_start
            anchors.horizontalCenter: parent.horizontalCenter
            width: 35
            text: "10"
        }
        Text {
            anchors.horizontalCenter: parent.horizontalCenter
            text: "WEIGHT END (+):"
        }
        TextField {
            id: weight_endp
            anchors.horizontalCenter: parent.horizontalCenter
            width: 35
            text: "10"
        }
        Text {
            anchors.horizontalCenter: parent.horizontalCenter
            text: "WEIGHT END (-):"
        }
        TextField {
            id: weight_endn
            anchors.horizontalCenter: parent.horizontalCenter
            width: 35
            text: "5"
        }
        Text {
            anchors.horizontalCenter: parent.horizontalCenter
            text: "MAX BADNESS:"
        }
        TextField {
            id: max_badness
            anchors.horizontalCenter: parent.horizontalCenter
            width: 35
            text: "100"
        }
    }

    QProcess { id: proc1 }
    QProcess { id: proc2 }

    property string black : "#000000"

    function applyToNotesInSelection(func) {
        var cursor = curScore.newCursor();
        cursor.rewind(1);
        var startStaff  = cursor.staffIdx;
        cursor.rewind(2);
        var endStaff   = cursor.staffIdx;
        var endTick    = cursor.tick // if no selection, end of score
        var fullScore = false;
        if (!cursor.segment) { // no selection
            fullScore = true;
            startStaff = 0; // start with 1st staff
            endStaff = curScore.nstaves - 1; // and end with last
        }
        console.log(startStaff + " - " + endStaff + " - " + endTick)
        for (var staff = startStaff; staff <= endStaff; staff++) {
            for (var voice = 0; voice < 4; voice++) {
                cursor.rewind(1); // sets voice to 0
                cursor.voice = voice; //voice has to be set after goTo
                cursor.staffIdx = staff;

                if (fullScore)
                cursor.rewind(0) // if no selection, beginning of score

                while (cursor.segment && (fullScore || cursor.tick < endTick)) {
                    if (cursor.element && cursor.element.type == Element.CHORD) {
                        var notes = cursor.element.notes;
                        for (var i = 0; i < notes.length; i++) {
                            var note = notes[i];
                            func(note);
                        }
                    }
                    cursor.next();
                }
            }
        }
    }

    function uncolorNote(note) {
        note.color = black;
    }

}
